#-------------------------------------------------------------
# Tcl code to generate a GUI that will run wsbeta
# to produce a psk file of single particle orbitals vs
# rotational frequency as a function of deformation.
#-------------------------------------------------------------
# D. Appelbe Thu Mar 14 10:16:55 2002
#-------------------------------------------------------------
set frame        QPB
set title        "Theory Codes: Quasi-Particle (Beta)"
set icon         qpb

set directory    $env(HOME)
set filename     ""
set tempfilename "qpb.com"

set z            20
set n            30
set b2           0.1
set b3           0.0
set b4           0.0
set b5           0.0
set b6           0.0

set execute      ""
set response     ""

set reportflag   0
set abortflag    0

#-------------------------------------------------------------
proc abort_report {} {
   global execute response abortflag

   set-footer "program terminated abnormally"
   set message "program terminated abnormally\n\
      \nexecuted: \"$execute\"\
      \nresponse received: \"$response\"\
      "
   if {[string length $message] < 1000} then {
      set abortflag [midas-confirmation "$message" "continue" "abort"]
   } else {
      midas-information "$message"
      set abortflag [midas-confirmation "report has been placed in separate window because of its size" "continue" "abort"]
   }
}

proc progress_report {w} {
   global execute response reportflag

   if {$reportflag == 0} then return

   if {$w == 0} then {
      midas-report "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   } else {
      midas-information "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   }
}

proc click-browse args {
    global env directory filename
#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{All Files}  *}
    }

    set file [tk_getSaveFile -parent [self frame-name] -title "Output File Selection"  \
       -initialdir $directory -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file == ""} then {return}

    set directory [file dirname $file]
    :modify DIRECTORY :value $directory
    set filename [file tail $file]
    :modify FILENAME :value $filename
}

proc click-quit {args} {
   global preferences
   global directory filename

#    save all task preferences
    set preferences(directory) $directory
    set preferences(filename) $filename
    update_preferences

    DeleteTask
}

proc click-reset { args } {
    global env directory filename z n b2 b3 b4 b5 b6

    :modify Z         :value 35
    :modify N         :value 35
    :modify LIST      :empty T
    :modify B2        :value 0.2
    :modify B3        :value 0.0
    :modify B4        :value 0.0
    :modify B5        :value 0.0
    :modify B6        :value 0.0

}

proc click-run {args} {
    global env directory filename z n b2 b3 b4 b5 b6 tempfilename
    global execute response reportflag abortflag

    insert-list "Program options are:-"
    insert-list "Directory = $directory"
    insert-list "Filename = $filename"
    insert-list "Z = $z"
    insert-list "N = $n"
    insert-list "beta2 = $b2"
    insert-list "beta3 = $b3"
    insert-list "beta4 = $b4"
    insert-list "beta5 = $b5"
    insert-list "beta6 = $b6"

    if {[midas-check-for-temp] == 0} then {return}
    set rc [catch {open $tempfilename w} f]
    if {$rc != 0} then {midas-warning "Unable to write temporary file\n$f"; return}

    set n1 "_n.psk"
    set p1 "_p.psk"

    puts $f "#!/bin/sh "
    puts $f ""
    puts $f "#==============================================================="
    puts $f "# Woods-Saxon Cranking with beta2,...beta6"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "#==============================================================="
    puts $f "# Name output directory for readable output - Try to make it "
    puts $f "#==============================================================="
    puts $f ""
    puts $f "OUTPUT_DIRECTORY='$directory'"
    puts $f ""
    puts $f "#==============================================================="
    puts $f "# Name output PSSKETCH files"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "NEUTRON_PLOT_FILE='$filename$n1'"
    puts $f "PROTON_PLOT_FILE='$filename$p1'"
    puts $f ""
    puts $f "#==============================================================="
    puts $f "#                     <<<<<<<<<<<< swbeta >>>>>>>>>>>>>"
    puts $f "#==============================================================="
    puts $f "# Change deformation parameters and proton (IZ) and neutron (IN) number"
    puts $f "#"
    puts $f "# NAZWIS defines the Woods-Saxon parameter set..."
    puts $f "#"
    puts $f "#        NAZWIS   -  IF (NAZWIS.EQ.0) WAHLBORN PARAMETERS,"
    puts $f "#                    IF (NAZWIS.EQ.1) ROST PARAMETERS,"
    puts $f "#                    IF (NAZWIS.EQ.2) CHEPURNOV PARAMETERS,"
    puts $f "#                    IF (NAZWIS.EQ.3) OPTIMIZED PARAMETERS,"
    puts $f "#                    IF (NAZWIS.EQ.4) UNIVERSAL PARAMETERS (default)"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "echo \"Running swbeta\""
    puts $f "rm -f fort.*"
    puts $f "rm -f \$OUTPUT_DIRECTORY/swbeta.out"
    puts $f "/home/dea/Bin/swbeta<<DONE>>\$OUTPUT_DIRECTORY/swbeta.out"
    puts $f " &INTDAT IAUTO=1, NAZWIS=4,IQUADR=1,IDECOM=1 &"
    puts $f " &DYNAMC IZ=$z, IN=$n "
    puts $f "  BETA2=$b2, "
    puts $f "  BETA3=$b3,"
    puts $f "  BETA4=$b4,"
    puts $f "  BETA5=$b5,"
    puts $f "  BETA6=$b6 &"
    puts $f "DONE"
    puts $f "mv -f fort.10 swbeta.dat"
    puts $f ""
    puts $f "#==============================================================="
    puts $f "#                    <<<<<<<<<<<< jxbeta >>>>>>>>>>>>>"
    puts $f "#==============================================================="
    puts $f "# Change proton and neutron numbers again: NNND"
    puts $f "# Warning: DO NOT touch NCUTDN, NCUTGN, NCUTDP, NCUTGP!"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "echo \"Running jxbeta for neutrons\""
    puts $f "mv -f swbeta.dat fort.10"
    puts $f "rm -f fort.*"
    puts $f "rm -f \$OUTPUT_DIRECTORY/jxbeta.out"
    puts $f "/home/dea/Bin/jxbeta<<NEUTRONS>>\$OUTPUT_DIRECTORY/jxbeta.out"
    puts $f " &PHYSIN IVAR=0, "
    puts $f " NCUTD=10,      "
    puts $f " NCUTG=100,      "
    puts $f " NNND=$n &          "
    puts $f " &AUXILI MAGMOM=1 & "
    puts $f "NEUTRONS"
    puts $f "mv -f fort.12 jxbeta_n.dat"
    puts $f ""
    puts $f "echo \"Running jxbeta for protons\""
    puts $f "/home/dea/Bin/jxbeta<<PROTONS>>\$OUTPUT_DIRECTORY/jxbeta.out"
    puts $f " &PHYSIN IVAR=1, "
    puts $f " NCUTD=10,      "
    puts $f " NCUTG=70,      "
    puts $f " NNND=$z &          "
    puts $f " &AUXILI MAGMOM=1 & "
    puts $f "PROTONS"
    puts $f "mv -f fort.12 jxbeta_p.dat"
    puts $f ""
    puts $f "#==============================================================="
    puts $f "#                      <<<<<<<<<<<< hfbc >>>>>>>>>>>>>"
    puts $f "#==============================================================="
    puts $f "# Crank here: STEPOM = frequency step, ILOM = No. of steps"
    puts $f "#"
    puts $f "# Pairing"
    puts $f "# (1) delta = DELTA0 * delta(BCS)   for DELTA0 > 0"
    puts $f "# (2) delta = abs(DELTA0)           for DELTA0 < 0 and IMODEL = 2"
    puts $f "# (3) delta = function(omega)       for DELTA0 = 0 and IMODEL = 2"
    puts $f "#            (then OMCRIT = frequency of 50% pairing) "
    puts $f "#==============================================================="
    puts $f ""
    puts $f "echo \"Running hfbc for neutrons. Please be patient\""
    puts $f "rm -f \$OUTPUT_DIRECTORY/hfbc.out"
    puts $f "rm -f fort.*"
    puts $f "mv -f jxbeta_n.dat fort.11"
    puts $f "/home/dea/Bin/hfbc<<NEUTRONS>>\$OUTPUT_DIRECTORY/hfbc.out"
    puts $f " &DEFINE IPROJ=0, IPRINT=1, IPLOT=2, IMODEL=2 &      "
    puts $f " &STARTI OMEGIN=0.001, "
    puts $f " STEPOM=0.05,           "
    puts $f " ILOM=21,               "
    puts $f " DELG=1.4,              "
    puts $f " STD=0.07,              "
    puts $f "  OMCRIT=0.70,           "
    puts $f "  DELTA0=0.0,           "
    puts $f "  YLAMM=10.0 &          "
    puts $f " &CONFIG NQPPPS(1)=0, NQPPPS(2)=0, NQPPPS(3)=0,"
    puts $f "         NQPPNS(1)=0, NQPPNS(2)=0, NQPPNS(3)=0,"
    puts $f "         NQNPPS(1)=0, NQNPPS(2)=0, NQNPPS(3)=0,"
    puts $f "         NQNPNS(1)=0, NQNPNS(2)=0, NQNPNS(3)=0     &"
    puts $f "NEUTRONS"
    puts $f "mv -f fort.25 spag_n.dat"
    puts $f ""
    puts $f "echo \"Running hfbc for protons. Please be patient\""
    puts $f "mv -f jxbeta_p.dat fort.11"
    puts $f "/home/dea/Bin/hfbc<<PROTONS>>\$OUTPUT_DIRECTORY/hfbc.out"
    puts $f " &DEFINE IPROJ=0, IPRINT=1, IPLOT=2, IMODEL=2 &"
    puts $f " &STARTI OMEGIN=0.001, "
    puts $f " STEPOM=0.05,           "
    puts $f " ILOM=21,               "
    puts $f " DELG=1.3,              "
    puts $f " STD=0.07,              "
    puts $f "  OMCRIT=0.70,           "
    puts $f "  DELTA0=0.0,           "
    puts $f "  YLAMM=10.0 &          "
    puts $f " &CONFIG NQPPPS(1)=0, NQPPPS(2)=0, NQPPPS(3)=0,"
    puts $f "         NQPPNS(1)=0, NQPPNS(2)=0, NQPPNS(3)=0,"
    puts $f "         NQNPPS(1)=0, NQNPPS(2)=0, NQNPPS(3)=0,"
    puts $f "         NQNPNS(1)=0, NQNPNS(2)=0, NQNPNS(3)=0     &"
    puts $f "PROTONS"
    puts $f "mv -f fort.25 spag_p.dat"
    puts $f ""
    puts $f "#==============================================================="
    puts $f "#                    <<<<<<<<<<<< spag2psk >>>>>>>>>>>>>"
    puts $f "#==============================================================="
    puts $f "# Produce PSSKETCH plot files"
    puts $f "# y scale defined by YMIN, YMAX, YSTEP"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "echo \"Producing plot files\""
    puts $f "rm -f \$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f "rm -f fort.*"
    puts $f "mv -f spag_n.dat fort.25"
    puts $f "/home/dea/Bin/spag2psk<<NEUTRONS>>\$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f " &INPUT INDAT=0,"
    puts $f "        YMIN=-0.5,"
    puts $f "        YMAX=2,"
    puts $f "        YSTEP=0.5 &"
    puts $f "NEUTRONS"
    puts $f "mv -f fort.9 \$OUTPUT_DIRECTORY/\$NEUTRON_PLOT_FILE"
    puts $f "echo \"Neutron plot file: \$OUTPUT_DIRECTORY/\$NEUTRON_PLOT_FILE\""
    puts $f ""
    puts $f "mv -f spag_p.dat fort.25"
    puts $f "/home/dea/Bin/spag2psk<<PROTONS>>\$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f " &INPUT INDAT=0,"
    puts $f "        YMIN=-0.5,"
    puts $f "        YMAX=2,"
    puts $f "        YSTEP=0.5 &"
    puts $f "PROTONS"
    puts $f "mv -f fort.9 \$OUTPUT_DIRECTORY/\$PROTON_PLOT_FILE"
    puts $f "echo \"Proton plot file:\$OUTPUT_DIRECTORY/\$PROTON_PLOT_FILE \""
    puts $f ""
    puts $f "#==============================================================="
    puts $f "# Delete big files - the only useful ones are the .out files"
    puts $f "#==============================================================="
    puts $f ""
    puts $f "rm -f fort.*"

    close $f

    set-footer "running code"

    set execute "exec chmod +x $tempfilename"
    set rc [catch { exec chmod +x $tempfilename } m]

    set response "$m"
    if {$rc != 0} then {
	  abort_report
      }
      progress_report 0

      insert-list $m

      set execute "exec ./$tempfilename"
      set rc [catch { exec ./$tempfilename } m]

      set response "$m"
      if {$rc != 0} then {
	  abort_report
      }
      progress_report 0

      insert-list $m

      clear-footer

}

proc insert-list {s} {:modify LIST :strings "$s\n"}

proc click arg {
    global debug
    global env directory filename z n b2 b3 b4 b5 b6

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   switch $w {
      DIRECTORY     {set directory $v}
      FILENAME      {set filename $v}
      Z             {set z $v}
      N             {set n $v}
      B2            {set b2 $v}
      B3            {set b3 $v}
      B4            {set b4 $v}
      B5            {set b5 $v}
      B6            {set b6 $v}
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p [list $w $v $i]} else {set-footer "no handler for widget $w"}
      }
   }
}

proc start args {
   global started preferences
   global directory filename
   if {$started == 0} then {
      make-frame
      set started 1
   } else {
      open-frame
   }
}

proc make-frame {} {
    global env directory filename z n b2 b3 b4 b5 b6

    set rows 8
    set cols 50

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
	    :icon-name "[self icon-title]" :resize T
    :panel P1 :layout T :padx 10 :pady 1
     :layout V
      :message M1 :label "This code generates, for a specific set of beta values, a plot for Neutrons and protons"
      :message M2 :label "of quasi particle levels vs rotational frequency"
      :message M3 :label "The output files are formatted for pssketch"
    :panel P2 :layout T :padx 10 :pady 5
     :layout H; :col-gap 10; :row-gap 10
     :text DIRECTORY  :label "Output Directory" :w 32 :valuex 15c :value $directory
     :next-row T
     :text FILENAME   :label "Output Filename"  :w 16 :valuex 15c :value $filename
     :hskip 3
     :button BROWSE
     :layout V
     :next-row T
     :layout H
     :number Z :label "Z" :min 10 :max 109 :w 5 :value $z :valuex 15c
     :number N :label "N" :min 10 :max 200 :w 5 :value $n :valuex 7c
     :next-row T
     :number B2 :label "beta_2" :min -1.0 :max 1.0 :inc 0.01 :w 5 :value $b2 :valuex 15c
     :number B3 :label "beta_3" :min -1.0 :max 1.0 :inc 0.01 :w 5 :value $b3 :valuex 7c
     :number B4 :label "beta_4" :min -1.0 :max 1.0 :inc 0.001 :w 5 :value $b4 :valuex 7c
     :next-row T
     :number B5 :label "beta_5" :min -1.0 :max 1.0 :inc 0.001 :w 5 :value $b5 :valuex 15c
     :number B6 :label "beta_6" :min -1.0 :max 1.0 :inc 0.001 :w 5 :value $b6 :valuex 7c
    :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 30; :row-gap 10
     :button RUN
     :button VIEW
     :button ADVANCED
     :button RESET
     :button EMPTY :label "Clear Log" :command ":modify LIST :empty T"
     :button QUIT

    :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
     :galley LIST :cols $cols :rows $rows :deaf T  :font variable
     open-frame
     :galley_size LIST $rows $cols

}