#-------------------------------------------------------------
# Tcl code to generate a GUI that will run wsbeta
# to produce a psk file of single particle orbitals vs
# rotational frequency as a function of deformation.
#-------------------------------------------------------------
# D. Appelbe Thu Mar 14 15:47:47 2002
#-------------------------------------------------------------
set frame        QPG
set title        "Theory Codes: Quasi-Particle (Beta, Gamma)"
set icon         qpg

set directory    $env(HOME)
set filename     ""
set tempfilename "spg.com"

set z            76
set n            92
set b2           0.17
set b4           0.0
set g            0.0

set execute      ""
set response     ""

set reportflag   0
set abortflag    0

#-------------------------------------------------------------
proc abort_report {} {
   global execute response abortflag

   set-footer "program terminated abnormally"
   set message "program terminated abnormally\n\
      \nexecuted: \"$execute\"\
      \nresponse received: \"$response\"\
      "
   if {[string length $message] < 1000} then {
      set abortflag [midas-confirmation "$message" "continue" "abort"]
   } else {
      midas-information "$message"
      set abortflag [midas-confirmation "report has been placed in separate window because of its size" "continue" "abort"]
   }
}

proc progress_report {w} {
   global execute response reportflag

   if {$reportflag == 0} then return

   if {$w == 0} then {
      midas-report "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   } else {
      midas-information "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   }
}

proc click-browse args {
    global env directory filename
#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{All Files}  *}
    }

    set file [tk_getSaveFile -parent [self frame-name] -title "Output File Selection"  \
       -initialdir $directory -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file == ""} then {return}

    set directory [file dirname $file]
    :modify DIRECTORY :value $directory
    set filename [file tail $file]
    :modify FILENAME :value $filename
}

proc click-quit {args} {
   global preferences
   global directory filename

#    save all task preferences
    set preferences(directory) $directory
    set preferences(filename) $filename
    update_preferences

    DeleteTask
}

proc click-reset { args } {
    global env directory filename z n b2 b4 g

    :modify Z         :value 35
    :modify N         :value 35
    :modify LIST      :empty T
    :modify B2        :value 0
    :modify B4        :value 0
    :modify G         :value 0

}

proc click-run {args} {
    global env directory filename z n b2 b4 g tempfilename
    global execute response reportflag abortflag

    insert-list "Program options are:-"
    insert-list "Directory = $directory"
    insert-list "Filename = $filename"
    insert-list "Z = $z"
    insert-list "N = $n"
    insert-list "beta2 = $b2"
    insert-list "beta4 = $b4"
    insert-list "Gamma = $g"

    if {[midas-check-for-temp] == 0} then {return}
    set rc [catch {open $tempfilename w} f]
    if {$rc != 0} then {midas-warning "Unable to write temporary file\n$f"; return}

    set n1 "_n.psk"
    set p1 "_p.psk"

    puts $f "#!/bin/sh"
    puts $f "# Autogenerated file by qpb"
    puts $f "# Produces WS single-particle diagrams for a given set of betas"
    puts $f "OUTPUT_DIRECTORY='$directory'"
    puts $f "NEUTRON_PLOT_FILE='$filename$n1'"
    puts $f "PROTON_PLOT_FILE='$filename$p1'"
    puts $f "cd \$OUTPUT_DIRECTORY/"
    puts $f "echo \"Running swgamma\""
    puts $f "rm -f \$OUTPUT_DIRECTORY/fort.10"
    puts $f "rm -f \$OUTPUT_DIRECTORY/swgamma.out"
    puts $f "swgamma<<DONE>>\$OUTPUT_DIRECTORY/swgamma.out"
    puts $f " &INTDAT IAUTO=1, NAZWIS=4 &"
    puts $f " &DYNAMC BETA2=$b2,BETA4=$b4,GAMM=$g,IZ=$z,IN=$n &"
    puts $f "DONE"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.10 \$OUTPUT_DIRECTORY/swgamma.dat"
    puts $f "echo \"Running jxgamma for neutrons\""
    puts $f "mv -f \$OUTPUT_DIRECTORY/swgamma.dat fort.10"
    puts $f "rm -f \$OUTPUT_DIRECTORY/fort.12"
    puts $f "rm -f \$OUTPUT_DIRECTORY/jxgamma.out"
    puts $f "jxgamma<<NEUTRONS>>\$OUTPUT_DIRECTORY/jxgamma.out"
    puts $f " &PHYSIN IVAR=0, "
    puts $f " ISETS=0,        "
    puts $f " NCUTDN=10,      "
    puts $f " NCUTGN=100,      "
    puts $f " NNND=$n &          "
    puts $f " &AUXILI IDECOM=1 & "
    puts $f "NEUTRONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.12 \$OUTPUT_DIRECTORY/jxgamma_n.dat"
    puts $f "echo \"Running jxgamma for protons\""
    puts $f "jxgamma<<PROTONS>>\$OUTPUT_DIRECTORY/jxgamma.out"
    puts $f " &PHYSIN IVAR=1, "
    puts $f " ISETS=0,        "
    puts $f " NCUTDP=10,      "
    puts $f " NCUTGP=70,      "
    puts $f " NNND=$z &          "
    puts $f " &AUXILI IDECOM=1 & "
    puts $f "PROTONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.12 \$OUTPUT_DIRECTORY/jxgamma_p.dat"
    puts $f "echo \"Running hfbc for neutrons. Please be patient\""
    puts $f "rm -f \$OUTPUT_DIRECTORY/hfbc.out"
    puts $f "rm -f \$OUTPUT_DIRECTORY/fort.25"
    puts $f "mv -f \$OUTPUT_DIRECTORY/jxgamma_n.dat \$OUTPUT_DIRECTORY/fort.11"
    puts $f "hfbc<<NEUTRONS>>\$OUTPUT_DIRECTORY/hfbc.out"
    puts $f " &DEFINE IPROJ=0, IPRINT=1, IPLOT=2, IMODEL=2 &      "
    puts $f " &STARTI OMEGIN=0.001, "
    puts $f " STEPOM=0.05,           "
    puts $f " ILOM=21,               "
    puts $f " DELG=1.4,              "
    puts $f " STD=0.07,              "
    puts $f "  OMCRIT=0.70,           "
    puts $f "  DELTA0=0.0,           "
    puts $f "  YLAMM=10.0 &          "
    puts $f " &CONFIG NQPPPS(1)=0, NQPPPS(2)=0, NQPPPS(3)=0,"
    puts $f "         NQPPNS(1)=0, NQPPNS(2)=0, NQPPNS(3)=0,"
    puts $f "         NQNPPS(1)=0, NQNPPS(2)=0, NQNPPS(3)=0,"
    puts $f "         NQNPNS(1)=0, NQNPNS(2)=0, NQNPNS(3)=0     &"
    puts $f "NEUTRONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.25 \$OUTPUT_DIRECTORY/spag_n.dat"
    puts $f "echo \"Running hfbc for protons. Please be patient\""
    puts $f "mv -f \$OUTPUT_DIRECTORY/jxgamma_p.dat \$OUTPUT_DIRECTORY/fort.11"
    puts $f "hfbc<<PROTONS>>\$OUTPUT_DIRECTORY/hfbc.out"
    puts $f " &DEFINE IPROJ=0, IPRINT=1, IPLOT=2, IMODEL=2 &"
    puts $f " &STARTI OMEGIN=0.001, "
    puts $f " STEPOM=0.05,           "
    puts $f " ILOM=21,               "
    puts $f " DELG=1.3,              "
    puts $f " STD=0.07,              "
    puts $f "  OMCRIT=0.70,           "
    puts $f "  DELTA0=0.0,           "
    puts $f "  YLAMM=10.0 &          "
    puts $f " &CONFIG NQPPPS(1)=0, NQPPPS(2)=0, NQPPPS(3)=0,"
    puts $f "         NQPPNS(1)=0, NQPPNS(2)=0, NQPPNS(3)=0,"
    puts $f "         NQNPPS(1)=0, NQNPPS(2)=0, NQNPPS(3)=0,"
    puts $f "         NQNPNS(1)=0, NQNPNS(2)=0, NQNPNS(3)=0     &"
    puts $f "PROTONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.25 \$OUTPUT_DIRECTORY/spag_p.dat"
    puts $f "echo \"Producing plot files\""
    puts $f "rm -f \$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f "rm -f \$OUTPUT_DIRECTORY/fort.9"
    puts $f "mv -f \$OUTPUT_DIRECTORY/spag_n.dat \$OUTPUT_DIRECTORY/fort.25"
    puts $f "spag2psk<<NEUTRONS>>\$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f " &input indat=0,"
    puts $f "        ymin=-0.5,"
    puts $f "        ymax=2,"
    puts $f "        ystep=0.5 &"
    puts $f "NEUTRONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.9 \$OUTPUT_DIRECTORY/\$NEUTRON_PLOT_FILE"
    puts $f "echo \"Neutron plot file: \$OUTPUT_DIRECTORY/\$NEUTRON_PLOT_FILE\""
    puts $f "mv -f \$OUTPUT_DIRECTORY/spag_p.dat \$OUTPUT_DIRECTORY/fort.25"
    puts $f "spag2psk<<PROTONS>>\$OUTPUT_DIRECTORY/spag2psk.out"
    puts $f " &input indat=0,"
    puts $f "        ymin=-0.5,"
    puts $f "        ymax=2,"
    puts $f "        ystep=0.5 &"
    puts $f "PROTONS"
    puts $f "mv -f \$OUTPUT_DIRECTORY/fort.9 \$OUTPUT_DIRECTORY/\$PROTON_PLOT_FILE"
    puts $f "echo \"Proton plot file:  \$OUTPUT_DIRECTORY/\$PROTON_PLOT_FILE \""
    puts $f "rm -f \$OUTPUT_DIRECTORY/fort.*"

    close $f

    set-footer "running code"

    set execute "exec chmod +x $tempfilename"
    set rc [catch { exec chmod +x $tempfilename } m]

    set response "$m"
    if {$rc != 0} then {
	  abort_report
      }
      progress_report 0

      insert-list $m

      set execute "exec ./$tempfilename"
      set rc [catch { exec ./$tempfilename } m]

      set response "$m"
      if {$rc != 0} then {
	  abort_report
      }
      progress_report 0

      insert-list $m

      clear-footer

}

proc insert-list {s} {:modify LIST :strings "$s\n"}

proc click arg {
    global debug
    global env directory filename z n b2 b4 g

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   switch $w {
      DIRECTORY     {set directory $v}
      FILENAME      {set filename $v}
      Z             {set z $v}
      N             {set n $v}
      B2            {set b2 $v}
      B4            {set b4 $v}
      G             {set g $v}
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p [list $w $v $i]} else {set-footer "no handler for widget $w"}
      }
   }
}

proc start args {
   global started preferences
   global directory filename
   if {$started == 0} then {
      make-frame
      set started 1
   } else {
      open-frame
   }
}

proc make-frame {} {
    global env directory filename z n b2 b4 g

    set rows 8
    set cols 50

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
	    :icon-name "[self icon-title]" :resize T
    :panel P1 :layout T :padx 10 :pady 1
     :layout V
      :message M1 :label "This code generates, for a specific set of beta(2,4) and gamma values, a plot for Neutrons and protons"
      :message M2 :label "of single particle levels vs rotational frequency"
      :message M3 :label "The output files are formatted for pssketch"
    :panel P2 :layout T :padx 10 :pady 5
     :layout H; :col-gap 10; :row-gap 10
     :text DIRECTORY  :label "Output Directory" :w 32 :valuex 15c :value $directory
     :next-row T
     :text FILENAME   :label "Output Filename"  :w 16 :valuex 15c :value $filename
     :hskip 3
     :button BROWSE
     :layout V
     :next-row T
     :layout H
     :number Z :label "Z" :min 10 :max 109 :w 5 :value $z :valuex 15c
     :number N :label "N" :min 10 :max 200 :w 5 :value $n :valuex 7c
     :next-row T
     :number B2 :label "beta_2" :min -1.0 :max 1.0 :inc 0.01 :w 5 :value $b2 :valuex 15c
     :number B4 :label "beta_4" :min -1.0 :max 1.0 :inc 0.001 :w 5 :value $b4 :valuex 7c
     :number G :label "gamma" :min -120.0 :max 120.0 :w 5 :value $g :valuex 7c
    :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 30; :row-gap 10
     :button RUN
     :button VIEW
     :button ADVANCED
     :button RESET
     :button EMPTY :label "Clear Log" :command ":modify LIST :empty T"
     :button QUIT

    :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
     :galley LIST :cols $cols :rows $rows :deaf T  :font variable
     open-frame
     :galley_size LIST $rows $cols

}