#    Duncan's Theory Code

set frame TheoryCodes
set title "Theory Codes: Woods-Saxon Beta"
set icon Nilsson


set directory $env(HOME)
set filename ""

set tempfilename "wsb"

## Added by DEA more variables
##set shape 0
##set mizol 1
##set nmaxp 12
##set nmaxn 14

set steps_menu {}
for {set i 1} {$i < 20} {incr i} {lappend steps_menu $i}
set increment_menu [list 0.1 0.2 0.5]
set start_menu {}
for {set i -10} {$i < 10} {incr i} {lappend start_menu [expr ($i*1.0)/10]}

set z 24
set n 32
set steps [first $steps_menu]
set increment [first $increment_menu]
set start [first $start_menu]

set execute ""
set response ""

set reportflag 0
set abortflag 0

proc abort_report {} {
   global execute response abortflag

   set-footer "program terminated abnormally"
   set message "program terminated abnormally\n\
      \nexecuted: \"$execute\"\
      \nresponse received: \"$response\"\
      "
   if {[string length $message] < 1000} then {
      set abortflag [midas-confirmation "$message" "continue" "abort"]
   } else {
      midas-information "$message"
      set abortflag [midas-confirmation "report has been placed in separate window because of its size" "continue" "abort"]
   }
}

proc progress_report {w} {
   global execute response reportflag

   if {$reportflag == 0} then return

   if {$w == 0} then {
      midas-report "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   } else {
      midas-information "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   }
}

proc click-browse args {
    global env directory filename
#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{All Files}  *}
    }

    set file [tk_getSaveFile -parent [self frame-name] -title "Output File Selection"  \
       -initialdir $directory -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file == ""} then {return}

    set directory [file dirname $file]
    :modify DIRECTORY :value $directory
    set filename [file tail $file]
    :modify FILENAME :value $filename
}

proc click-advanced {args} {

   insert-list "The Advanced button is not implemented"
}

proc click-run {args} {
   global directory filename tempfilename
   global z n steps increment start
   global execute response reportflag abortflag
   global shape, mizol, nmaxp, nmaxn

  insert-list "Program options are:-"
  insert-list "Directory = $directory"
  insert-list "Filename = $filename"
  insert-list "Z = $z"
  insert-list "N = $n"
  insert-list "# of Steps = $steps"
  insert-list "Increment = $increment"
  insert-list "Start at beta_2 = $start"

#    make a temp file with the arguments

    if {[midas-check-for-temp] == 0} then {return}

    set rc [catch {open $tempfilename w} f]
    if {$rc != 0} then {midas-warning "Unable to write temporary file\n$f"; return}

    set a  "_n.dat"
    set a1 "_n.psk"
    set b  "_p.dat"
    set b1 "_p.psk"

    puts $f "#!/bin/sh"
    puts $f "# Autogenerated file by Swb"
    puts $f "# Produces WS Nilsson diagrams"
    puts $f "OUTPUT_DIRECTORY='$directory'"
    puts $f "NEUTRON_PLOT_FILE='$filename$a1'"
    puts $f "PROTON_PLOT_FILE='$filename$b1'"
    puts $f "rm -f fort.10"
    puts $f "rm -f \$OUTPUT_DIRECTORY/swbeta.out"
    puts $f "swbeta<<NEUTRONS>>\$OUTPUT_DIRECTORY/swbeta.out"
    puts $f " &intdat mizol=1, ishape=0 &"
    puts $f " &dynamc iz=$z, in=$n, nmaxp=12, nmaxn=14,"
    puts $f " beta2=$start, beta3=0.0, beta4=0.0, beta5=0.0, beta6=0.0,"
    puts $f "    iendd=0 &"
    for {set i 1} {$i < $steps} {incr i} { puts $f " &dynamc beta2=[expr ($start + ( $increment * $i ) )], beta4=0.0000 &"}
    puts $f " &dynamc beta2=[ expr ( $start + ($increment * ($steps - 1 ) ) ) ], beta4=0.000, iendd=1 &"
    puts $f "NEUTRONS"
    puts $f " mv -f fort.10 $directory/$filename$a"
    puts $f "rm -f fort.10"
    puts $f "swbeta<<PROTONS>>\$OUTPUT_DIRECTORY/swbeta.out"
    puts $f " &intdat mizol=2, ishape=0 &"
    puts $f " &dynamc iz=$z, in=$n, nmaxp=12, nmaxn=14,"
    puts $f " beta2=$start, beta3=0.0, beta4=0.0, beta5=0.0, beta6=0.0,"
    puts $f "    iendd=0 &"
    for {set i 1} {$i < $steps} {incr i} { puts $f " &dynamc beta2=[expr ($start + ( $increment * $i ) )], beta4=0.0000 &"}
    puts $f " &dynamc beta2=[ expr ( $start + ($increment * ($steps - 1 ) ) ) ], beta4=0.000, iendd=1 &"
    puts $f "PROTONS"
    puts $f " mv -f fort.10 $directory/$filename$b"
    puts $f "mv -f $directory/$filename$a fort.11"
    puts $f "rm -f fort.25"
    puts $f "rm -f $directory/swbplot.out"
    puts $f "swbplot<<NEUTRONS>>$directory/swbplot.out"
    puts $f "1,$steps,1,1,0,1"
    puts $f "NEUTRONS"
    puts $f "mv -f fort.25 $directory/$filename$a"
    puts $f "mv -f $directory/$filename$b fort.11"
    puts $f "rm -f fort.25"
    puts $f "swbplot<<PROTONS>>$directory/swbplot.out"
    puts $f "1,$steps,1,1,0,1"
    puts $f "PROTONS"
    puts $f "mv -f fort.25 $directory/$filename$b"
    puts $f "rm -f $directory/spag2psk.out"
    puts $f "mv -f $directory/$filename$a fort.25"
    puts $f "spag2psk<<NEUTRONS>>$directory/spag2psk.out"
    puts $f " &INPUT INDAT=1,"
    puts $f "        YMIN=-20,"
    puts $f "        YMAX=0,"
    puts $f "        YSTEP=2 &"
    puts $f "NEUTRONS"
    puts $f "mv -f fort.9 $directory/$filename$a1"
    puts $f "mv -f $directory/$filename$b fort.25"
    puts $f "spag2psk<<PROTONS>>$directory/spag2psk.out"
    puts $f " &INPUT INDAT=1,"
    puts $f "        YMIN=-10,"
    puts $f "        YMAX=10,"
    puts $f "        YSTEP=2 &"
    puts $f "PROTONS"
    puts $f "mv -f fort.9 $directory/$filename$b1"
    puts $f "rm -f $directory/*.out"
    puts $f "rm -f fort.*"

    close $f

      set-footer "running code"

      set execute "exec chmod +x $tempfilename"
      set rc [catch { exec chmod +x $tempfilename } m]

      set response "$m"
      if {$rc != 0} then {
         abort_report
#         if {$abortflag == 0} then {continue} else {break}
      }
      progress_report 0

      insert-list $m

      set execute "exec ./$tempfilename"
      set rc [catch { exec ./$tempfilename } m]

      set response "$m"
      if {$rc != 0} then {
         abort_report
#         if {$abortflag == 0} then {continue} else {break}
      }
      progress_report 0

      insert-list $m

      clear-footer

}

proc click-quit {args} {
   global preferences
   global directory filename

#    save all task preferences
    set preferences(directory) $directory
    set preferences(filename) $filename
    update_preferences

    DeleteTask
}

proc click-reset {args} {
   global directory filename
   global z n steps increment start

    :modify Z         :value 24
    :modify N         :value 32
    :modify STEPS     :value 0
    :modify INCREMENT :value 0
    :modify START     :value 10
    :modify LIST      :empty T

}

proc insert-list {s} {:modify LIST :strings "$s\n"}

proc click arg {
   global debug
   global directory filename
   global shape mizol nmaxp nmaxn
   global z n steps increment start

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   switch $w {
      DIRECTORY     {set directory $v}
      FILENAME      {set filename $v}
      Z             {set z $v}
      N             {set n $v}
      STEPS         {set steps $i}
      INCREMENT     {set increment $i}
      START         {set start $i}
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p [list $w $v $i]} else {set-footer "no handler for widget $w"}
      }
   }
}

proc start args {
   global started preferences
   global directory filename
   if {$started == 0} then {

#    load task preferences
      load_preferences
      if {[info exists preferences(directory)]} then {set directory $preferences(directory)}
      if {[info exists preferences(filename)]} then {set filename $preferences(filename)}

      make-frame
      set started 1
   } else {
      open-frame
   }
}

proc make-frame {} {
   global env
   global directory filename
   global z n steps increment start
   global steps_menu increment_menu start_menu
   global shape, mizol, nmaxp, nmaxn

  set rows 8
  set cols 50

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-name "[self icon-title]" :resize T

   :panel P1 :layout T :padx 10 :pady 5
     :layout V
       :message M1 :label "This code generates a 'Nilsson' plot."
       :message M2 :label "Two output pssketch format files are generated."
       :message M3 :label "Contact D.Appelbe for more information."

   :panel P2 :layout T :padx 10 :pady 5
     :layout H; :col-gap 10; :row-gap 10

     :text DIRECTORY  :label "Output Directory" :w 32 :valuex 15c :value $directory
     :next-row T
     :text FILENAME   :label "Output Filename"  :w 16 :valuex 15c :value $filename
     :hskip 50
     :button BROWSE
     :layout V
     :next-row T
#     :choice-stack Z         :label "Z"               :strings $z_menu :value [lsearch $z_menu $z] :valuex 15c
#     :choice-stack N         :label "N"               :strings $n_menu :value [lsearch $n_menu $n] :valuex 15c
     :number Z   :label "Z"  :min 5 :max 109 :w 4 :value $z :valuex 15c
     :number N   :label "N"  :min 5 :max 169 :w 4 :value $n :valuex 15c
     :choice-stack STEPS     :label "# of Steps"      :strings $steps_menu :value [lsearch $steps_menu $steps] :valuex 15c
     :choice-stack INCREMENT :label "Increment"       :strings $increment_menu :value [lsearch $increment_menu $increment] :valuex 15c
     :choice-stack START     :label "Start at beta_2" :strings $start_menu :value [lsearch $start_menu $start] :valuex 15c

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 30; :row-gap 10
     :button RUN
     :button VIEW
     :button ADVANCED
     :button RESET
     :button EMPTY :label "Clear log" :command ":modify LIST :empty T"
     :button QUIT

   :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
     :galley LIST :cols $cols :rows $rows :deaf T  :font variable
     open-frame
     :galley_size LIST $rows $cols
}
