#-------------------------------------------------------------
# Tcl code to generate a GUI that will run extract TRS plots
#-------------------------------------------------------------
# D. Appelbe Thu Mar 14 16:06:51 2002
#-------------------------------------------------------------
set frame        TRS
set title        "Theory Codes: Extract TRS"
set icon         trs

set directory    $env(HOME)
set filename     ""
set tempfilename "trs.com"

set mesh_menu {}
lappend mesh_menu "--"
lappend mesh_menu "Ba"
lappend mesh_menu "Nd"
lappend mesh_menu "Pt"
set mesh "--"

set z_menu {}
lappend z_menu "--"
set z "--"

set baz_menu {}
lappend baz_menu "38"
for { set i 44 } { $i < 63 } { lappend baz_menu $i }

set n_menu {}
lappend n_menu "--"
set n "--"

set qp_menu {}
for { set i 0 } { $i < 5 } { incr i } { lappend qp_menu $i }
set qp 0
set qn_menu {}
for { set i 0 } { $i < 5 } { incr i } { lappend qn_menu $i }
set qn 0

set qp1_menu {}
lappend qp1_menu "-"
lappend qp1_menu "a"
lappend qp1_menu "b"
lappend qp1_menu "c"
lappend qp1_menu "d"
lappend qp1_menu "e"
lappend qp1_menu "f"
lappend qp1_menu "g"
lappend qp1_menu "h"
set qp1 "-"

set qn1_menu {}
lappend qp1_menu "-"
lappend qp1_menu "A"
lappend qp1_menu "B"
lappend qp1_menu "C"
lappend qp1_menu "D"
lappend qp1_menu "E"
lappend qp1_menu "F"
lappend qp1_menu "G"
lappend qp1_menu "H"
set qn1 "-"


set execute      ""
set response     ""

set reportflag   0
set abortflag    0

#-------------------------------------------------------------
proc abort_report {} {
   global execute response abortflag

   set-footer "program terminated abnormally"
   set message "program terminated abnormally\n\
      \nexecuted: \"$execute\"\
      \nresponse received: \"$response\"\
      "
   if {[string length $message] < 1000} then {
      set abortflag [midas-confirmation "$message" "continue" "abort"]
   } else {
      midas-information "$message"
      set abortflag [midas-confirmation "report has been placed in separate window because of its size" "continue" "abort"]
   }
}

proc progress_report {w} {
   global execute response reportflag

   if {$reportflag == 0} then return

   if {$w == 0} then {
      midas-report "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   } else {
      midas-information "executed: \"$execute\"\
       \nresponse received: \"$response\"\
       "
   }
}

proc click-browse args {
    global env directory filename
#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{All Files}  *}
    }

    set file [tk_getSaveFile -parent [self frame-name] -title "Output File Selection"  \
       -initialdir $directory -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file == ""} then {return}

    set directory [file dirname $file]
    :modify DIRECTORY :value $directory
    set filename [file tail $file]
    :modify FILENAME :value $filename
}

proc click-quit {args} {
   global preferences
   global directory filename

#    save all task preferences
    set preferences(directory) $directory
    set preferences(filename) $filename
    update_preferences

    DeleteTask
}

proc click-reset { args } {
    global env directory filename z n mesh_menu
    global baz_menu ban_menu mesh
    global qp_menu qn_menu qp qn
    global qp1 qp1_menu qp2 qp2_menu qp3 qp3_menu qp4 qp4_menu
    global qn1 qn1_menu qn2 qn2_menu qn3 qn3_menu qn4 qn4_menu

    :modify MSH       :value 0
    :modify BAZ       :value 0
    :modify BAN       :value 0
    :modify QP        :value 0
    :modify QP1       :value 0
    :modify QP2       :value 0
    :modify QP3       :value 0
    :modify QP4       :value 0
    :modify QN        :value 0
    :modify QN1       :value 0
    :modify QN2       :value 0
    :modify QN3       :value 0
    :modify QN4       :value 0
    :modify LIST      :empty T

}

proc click-run {args} {
    global env directory filename z n tempfilename
    global execute response reportflag abortflag

    insert-list "Program options are:-"
    insert-list "Directory = $directory"
    insert-list "Filename = $filename"
    insert-list "Z = $z"
    insert-list "N = $n"

    if {[midas-check-for-temp] == 0} then {return}
    set rc [catch {open $tempfilename w} f]
    if {$rc != 0} then {midas-warning "Unable to write temporary file\n$f"; return}


    close $f

#    set-footer "running code"
#
#    set execute "exec chmod +x $tempfilename"
#    set rc [catch { exec chmod +x $tempfilename } m]
#
#    set response "$m"
#    if {$rc != 0} then {
#	  abort_report
#      }
#      progress_report 0
#
#      insert-list $m
#
#      set execute "exec ./$tempfilename"
#      set rc [catch { exec ./$tempfilename } m]
#
#      set response "$m"
#      if {$rc != 0} then {
#	  abort_report
#      }
#      progress_report 0
#
#      insert-list $m
#
#      clear-footer

}

proc insert-list {s} {:modify LIST :strings "$s\n"}

proc click arg {
    global debug
    global env directory filename z n mesh
    global qp qn

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   switch $w {
      DIRECTORY     {set directory $v}
      FILENAME      {set filename $v}
      Z             {set z $v}
      N             {set n $v}
      QP            {set qp $v}
      QN            {set qn $v}
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p [list $w $v $i]} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click-msh { arg } {
    global z n baz_menu ndz_menu ptz_menu

    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set i [lindex $arg 4]

    puts $arg

    set mesh $w
    puts $mesh
    if { $mesh == "Ba" } { :modify BAZ :strings $baz_menu }
    if { $mesh == "Nd" } { :modify BAZ :strings $ndz_menu }
    if { $mesh == "Pt" } { :modify BAZ :strings $ptz_menu }

}

proc start args {
   global started preferences
   global directory filename
   if {$started == 0} then {
      make-frame
      set started 1
   } else {
      open-frame
   }
}

proc make-frame {} {
    global env directory filename z n mesh_menu
    global z_menu n_menu mesh
    global qp_menu qn_menu qp qn
    global qp1 qp1_menu
    global qn1 qn1_menu

    set rows 8
    set cols 50

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
	    :icon-name "[self icon-title]" :resize T
    :panel P1 :layout T :padx 10 :pady 1
     :layout V
      :message M1 :label "This GUI can be used to extract TRS plots from the meshes available on the NPG SUNS"
    :panel P2 :layout T :padx 10 :pady 5
     :layout H; :col-gap 10; :row-gap 10
     :text DIRECTORY  :label "Output Directory" :w 32 :valuex 15c :value $directory
     :next-row T
     :text FILENAME   :label "Output Filename"  :w 16 :valuex 15c :value $filename
     :hskip 3
     :button BROWSE
     :layout V
     :next-row T
     :layout H
     :choice-stack MSH :label "Mesh" :strings $mesh_menu :value [lsearch $mesh_menu $mesh] :valuex 6c
     :choice-stack BAZ :label "Z" :strings $z_menu :value [lsearch $z_menu $z] :valuex 2c
     :hskip 10
     :choice-stack BAN :label "N" :strings $n_menu :value [lsearch $n_menu $n] :valuex 2c
     :next-row T
     :choice-stack QP :label "# QP" :strings $qp_menu :value [lsearch $qp_menu $qp] :valuex 6c
     :hskip 10
     :next-row T
     :choice-stack QN :label "# QN" :strings $qn_menu :value [lsearch $qn_menu $qn] :valuex 6c
     :hskip 10
    :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 30; :row-gap 10
     :button RUN
     :button VIEW
     :button ADVANCED
     :button RESET
     :button EMPTY :label "Clear Log" :command ":modify LIST :empty T"
     :button QUIT

    :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
     :galley LIST :cols $cols :rows $rows :deaf T  :font variable
     open-frame
     :galley_size LIST $rows $cols

}